library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

package cntr_pckg is

component cntr
	generic (
		LENGTH: natural		-- number of bits in counter
	);
    port (
    	rst: in STD_LOGIC;	-- synchronous reset
        clk: in STD_LOGIC;	-- counter clock
        cnt: out UNSIGNED(LENGTH-1 downto 0)	-- counter output
    );
end component;

end cntr_pckg;


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity cntr is
	generic (
		LENGTH: natural		-- number of bits in counter
	);
    port (
    	rst: in STD_LOGIC;	-- synchronous reset
        clk: in STD_LOGIC;	-- counter clock
        cnt: out UNSIGNED(LENGTH-1 downto 0)	-- counter output
    );
end cntr;

architecture cntr_arch of cntr is
signal cnt_r: UNSIGNED(LENGTH-1 downto 0);		-- counter register
begin
	COUNT: process(clk)
	begin
		-- change counter only on rising clock edges
		if (clk'event and clk='1') then
			if rst='1' then		-- synchronous reset to 0
				cnt_r <= TO_UNSIGNED(0,LENGTH);
			else				-- otherwise, increment counter
				cnt_r <= cnt_r + 1;
			end if;
		end if;
	end process COUNT;
	
	cnt <= cnt_r;	-- output register contents
end cntr_arch;

